package model

import (
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo/types"
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

const Collection = "product"

type Product struct {
	ID           primitive.ObjectID  `bson:"_id,omitempty" json:"id,omitempty"`
	StyleNumber  string              `bson:"styleNumber"`
	Title        ProductTitle        `bson:"title"`
	Description  string              `bson:"description"`
	Season       string              `bson:"season"`
	Brand        ProductBrand        `bson:"brand"`
	Care         string              `bson:"care"`
	MadeIn       string              `bson:"madeIn"`
	HsCode       string              `bson:"hsCode"`
	InfoSelect   string              `bson:"infoSelect"` // 商品信息默认选择哪个商品
	Colors       ProductColors       `bson:"colors"`
	Category     ProductCategory     `bson:"category"`
	Compositions ProductCompositions `bson:"compositions"`
	Fits         []string            `bson:"fits"`
	Genders      ProductGenders      `bson:"genders"`
	Models       []string            `bson:"models"`
	UpdateAt     time.Time           `bson:"updateAt,omitempty" json:"updateAt,omitempty"`
	CreateAt     time.Time           `bson:"createAt,omitempty" json:"createAt,omitempty"`
}

type Products []Product

type ProductGender string

type ProductGenders []ProductGender

const (
	GenderMan   ProductGender = "man"
	GenderWoman ProductGender = "woman"
	GenderBaby  ProductGender = "baby"
	GenderGirl  ProductGender = "girl"
	GenderBoy   ProductGender = "boy"
)

type ProductComposition struct {
	Part     string `bson:"part"`
	Material string `bson:"material"`
	Value    string `bson:"value"`
}

type ProductCompositions []ProductComposition

type ProductCategory struct {
	Id       string            `bson:"id"`
	Name     string            `bson:"name"`
	Children []ProductCategory `json:"children" bson:"children"`
}

func (receiver ProductCategory) GetIds() []string {
	ids := []string{receiver.Id}
	for _, child := range receiver.Children {
		ids = append(ids, child.GetIds()...)
	}
	return ids
}

type ProductColor struct {
	ColorCode  string           `bson:"colorCode"`
	Id         string           `bson:"id"`
	Name       string           `bson:"name"`
	BasicId    string           `bson:"basicId"`
	BasicName  string           `bson:"basicName"`
	InfoSelect string           `bson:"infoSelect"` // 商品信息默认选择哪个商品
	Skus       ProductColorSkus `bson:"skus"`
	Images     []string         `bson:"images"`
}

type ProductColors []ProductColor

type ProductColorSku struct {
	Pc3SkuId                         string                             `bson:"pc3SkuId"`
	Pc3ProductId                     string                             `bson:"pc3ProductId"`
	SourceId                         string                             `bson:"sourceId"`
	SiteId                           string                             `bson:"siteId"`
	Size                             string                             `bson:"size"`
	Stock                            int                                `bson:"stock"`
	Price                            float64                            `bson:"price"`
	Currency                         string                             `bson:"currency"`
	ProductColorSkuPlatformStoreSkus []ProductColorSkuPlatformStoreSkus `bson:"productColorSkuPlatformStoreSkus"`
}

type ProductColorSkus []ProductColorSku

type ProductTitle struct {
	En string `bson:"en"`
	Cn string `bson:"cn"`
}

type ProductBrand struct {
	Id   string `bson:"id"`
	Name string `bson:"name"`
}

type ProductColorSkuPlatformStoreSku struct {
	Platform       string              `bson:"platform"`             // 销售平台
	Store          string              `bson:"store"`                // 销售平台具体店铺
	Qty            int                 `bson:"qty"`                  // 库存
	FixedQty       *int                `bson:"fixedQty,omitempty"`   // 虚拟库存
	Route          string              `bson:"route"`                // 线路
	Currency       string              `bson:"currency"`             // 销售平台的币种
	OriginCurrency string              `bson:"originCurrency"`       // 原始币种
	Price          float64             `bson:"price"`                // 最终价格
	FixPrice       *float64            `bson:"fixPrice,omitempty"`   // 固定价格
	FixMargin      *float64            `bson:"fixMargin,omitempty"`  // 固定加价率(影响计价)
	Rate           float64             `bson:"rate"`                 // OriginCurrency -> Currency 的汇率
	CnyRate        float64             `bson:"cnyRate"`              // Currency -> CNY 的汇率
	Margin         float64             `bson:"margin"`               // 加价率
	SubjectRate    float64             `bson:"subjectRate"`          // 专题系数
	Subject        *primitive.ObjectID `bson:"subject"`              // 绑定的专题id
	DiscountRate   float64             `bson:"discountRate"`         // 商家系数
	TotalDiscount  float64             `bson:"totalDiscount"`        // 总折扣
	TaxRate        float64             `bson:"taxRate"`              // 税率
	Freight        float64             `bson:"freight"`              // 商家运费
	Express        float64             `bson:"express"`              // 跨境费
	ChangeRate     *float64            `bson:"changeRate,omitempty"` // 价格变动率, 0.1表示涨价10%, -0.1表示降价10%
	ChangeAt       *time.Time          `bson:"changeAt,omitempty"`   // 价格变动时间，方便筛选
	Memos          types.Memos         `bson:"memos,omitempty"`      // 操作备注
	// todo 有固定价格固定库存等操作就是有固定操作
	IsFixed *bool `bson:"isFixed,omitempty"` // 是否有固定操作
}

type ProductColorSkuPlatformStoreSkus []ProductColorSkuPlatformStoreSku
